'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = immutableStateInvariantMiddleware;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _lodashLangCloneDeep = require('lodash/lang/cloneDeep');

var _lodashLangCloneDeep2 = _interopRequireDefault(_lodashLangCloneDeep);

var _wasMutated = require('./wasMutated');

var _wasMutated2 = _interopRequireDefault(_wasMutated);

function isImmutableDefault(value) {
  return typeof value !== 'object';
}

function copyState(state, isImmutable) {
  return (0, _lodashLangCloneDeep2['default'])(state, function (value) {
    if (isImmutable(value)) {
      return value;
    }
  });
}

var BETWEEN_DISPATCHES_MESSAGE = ['A state mutation was detected between dispatches.', ' This may cause incorrect behavior. ', '(http://rackt.github.io/redux/docs/Troubleshooting.html#never-mutate-reducer-arguments)'].join('');

var INSIDE_DISPATCH_MESSAGE = ['A state mutation was detected inside a dispatch.', ' Take a look at the reducer(s) handling the action %s. ', '(http://rackt.github.io/redux/docs/Troubleshooting.html#never-mutate-reducer-arguments)'].join('');

function immutableStateInvariantMiddleware() {
  var isImmutable = arguments.length <= 0 || arguments[0] === undefined ? isImmutableDefault : arguments[0];

  return function (_ref) {
    var getState = _ref.getState;

    var lastStateRef = getState();
    var lastStateCopy = copyState(lastStateRef, isImmutable);

    return function (next) {
      return function (action) {
        var stateRef = getState();

        (0, _invariant2['default'])(!(0, _wasMutated2['default'])(lastStateRef, lastStateCopy, stateRef, isImmutable), BETWEEN_DISPATCHES_MESSAGE);

        var stateCopy = copyState(stateRef, isImmutable);
        var dispatchedAction = next(action);
        lastStateRef = getState();

        (0, _invariant2['default'])(!(0, _wasMutated2['default'])(stateRef, stateCopy, lastStateRef, isImmutable), INSIDE_DISPATCH_MESSAGE, action.type || action);

        lastStateCopy = copyState(lastStateRef, isImmutable);
        return dispatchedAction;
      };
    };
  };
}

module.exports = exports['default'];